@@ -24,11 +24,12 @@
 #include "dlg-add-to-catalog.h"
 #include "gth-catalog.h"
 #include "gth-file-source-catalogs.h"
+#include "preferences.h"
 
 
 #define GET_WIDGET(name) _gtk_builder_get_widget (data->builder, (name))
 #define ADD_TO_CATALOG_DIALOG_NAME "add-to-catalog"
-#define UPDATE_SELECTION_DELAY 100
+#define UPDATE_SELECTION_DELAY 50
 
 
 typedef struct {
@@ -85,12 +86,14 @@ typedef struct {
 	GtkBuilder    *builder;
 	GtkWidget     *dialog;
 	GtkWidget     *source_tree;
+	GtkWidget     *info;
 	AddData       *add_data;
 	GthFileSource *catalog_source;
 	GthFileData   *new_catalog;
 	GthFileData   *new_library;
 	gulong         file_selection_changed_event;
-	guint          update_selectection_event;
+	guint          update_selection_event;
+	GSettings     *settings;
 } DialogData;
 
 
@@ -104,14 +107,15 @@ destroy_cb (GtkWidget  *widget,
 					     data->file_selection_changed_event);
 		data->file_selection_changed_event = 0;
 	}
-	if (data->update_selectection_event != 0) {
-		g_source_remove (data->update_selectection_event);
-		data->update_selectection_event = 0;
+	if (data->update_selection_event != 0) {
+		g_source_remove (data->update_selection_event);
+		data->update_selection_event = 0;
 	}
 	add_data_unref (data->add_data);
 	_g_object_unref (data->catalog_source);
 	_g_object_unref (data->new_catalog);
 	_g_object_unref (data->new_library);
+	_g_object_unref (data->settings);
 	g_object_unref (data->builder);
 	g_free (data);
 }
@@ -158,13 +162,15 @@ catalog_save_done_cb (void     **buffer,
 				    add_data->files,
 				    GTH_MONITOR_EVENT_CREATED);
 
-	if (add_data->view_destination)
-		gth_browser_go_to (add_data->browser, add_data->catalog_file, NULL);
-
 	if (add_data->close_after_adding) {
+		if (add_data->view_destination)
+			gth_browser_go_to (add_data->browser, add_data->catalog_file, NULL);
+
 		if (add_data->dialog != NULL)
 			gtk_widget_destroy (add_data->dialog);
 	}
+	else
+		gth_browser_show_next_image (add_data->browser, FALSE, FALSE);
 
 	add_data_unref (add_data);
 }
@@ -181,14 +187,11 @@ catalog_ready_cb (GObject  *catalog,
 	gsize    length;
 	GFile   *gio_file;
 
-	if (error != NULL) {
-		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (add_data->parent_window), _("Could not add the files to the catalog"), error);
-		add_data_unref (add_data);
-		return;
-	}
-
 	_g_object_unref (add_data->catalog);
-	add_data->catalog = (GthCatalog *) catalog;
+	if (catalog != NULL)
+		add_data->catalog = (GthCatalog *) catalog;
+	else
+		add_data->catalog = gth_catalog_new_for_file (add_data->catalog_file);
 
 	for (scan = add_data->files; scan; scan = scan->next)
 		gth_catalog_insert_file (add_data->catalog, (GFile *) scan->data, -1);
@@ -223,6 +226,7 @@ static void
 add_selection_to_catalog (DialogData *data,
 			  gboolean    close_after_adding)
 {
+	char  *last_catalog;
 	GList *items;
 	GList *file_list;
 
@@ -231,6 +235,10 @@ add_selection_to_catalog (DialogData *data,
 	if (data->add_data->catalog_file == NULL)
 		return;
 
+	last_catalog = g_file_get_uri (data->add_data->catalog_file);
+	g_settings_set_string (data->settings, PREF_CATALOGS_LAST_CATALOG, last_catalog);
+	g_free (last_catalog);
+
 	_g_object_list_unref (data->add_data->files);
 	data->add_data->files = NULL;
 
@@ -252,24 +260,7 @@ static void
 add_button_clicked_cb (GtkWidget  *widget,
 		       DialogData *data)
 {
-	add_selection_to_catalog (data, FALSE);
-}
-
-
-static void
-add_close_button_clicked_cb (GtkWidget  *widget,
-			     DialogData *data)
-{
-	add_selection_to_catalog (data, TRUE);
-}
-
-
-static void
-source_tree_open_cb (GthFolderTree *folder_tree,
-		     GFile         *file,
-                     gpointer       user_data)
-{
-	add_button_clicked_cb (NULL, (DialogData *)user_data);
+	add_selection_to_catalog (data, ! gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("keep_open_checkbutton"))));
 }
 
 
@@ -278,22 +269,36 @@ update_sensitivity (DialogData *data)
 {
 	GFile    *selected_catalog;
 	GList    *items;
+	GList    *file_data_list;
 	gboolean  can_add;
 
 	selected_catalog = get_selected_catalog (data);
 	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (data->browser)));
 	can_add = (items != NULL) && (selected_catalog != NULL);
-	gtk_widget_set_sensitive (GET_WIDGET ("add_close_button"), can_add);
-	gtk_widget_set_sensitive (GET_WIDGET ("add_button"), can_add);
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK, can_add);
+	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (GET_WIDGET ("view_destination_checkbutton")), gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("keep_open_checkbutton"))));
+	gtk_widget_set_sensitive (GET_WIDGET ("view_destination_checkbutton"), ! gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("keep_open_checkbutton"))));
 
+	file_data_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (data->browser)), items);
+	gth_file_selection_info_set_file_list (GTH_FILE_SELECTION_INFO (data->info), file_data_list);
+
+	_g_object_list_unref (file_data_list);
 	_gtk_tree_path_list_free (items);
 	_g_object_unref (selected_catalog);
 }
 
 
+static void
+source_tree_changed_cb (GthVfsTree *folder_tree,
+			gpointer    user_data)
+{
+	update_sensitivity ((DialogData *) user_data);
+}
+
+
 static void
 source_tree_selection_changed_cb (GtkTreeSelection *treeselection,
-                                  gpointer          user_data)
+				  gpointer          user_data)
 {
 	update_sensitivity ((DialogData *) user_data);
 }
@@ -446,13 +451,13 @@ new_catalog_dialog_response_cb (GtkWidget *dialog,
 
 static void
 new_catalog_button_clicked_cb (GtkWidget  *widget,
-		       	       DialogData *data)
+			       DialogData *data)
 {
 	GtkWidget *dialog;
 
 	dialog = gth_request_dialog_new (GTK_WINDOW (data->dialog),
 					 GTK_DIALOG_MODAL,
-					 _("New catalog"),
+					 _("New Catalog"),
 					 _("Enter the catalog name:"),
 					 _GTK_LABEL_CANCEL,
 					 _("C_reate"));
@@ -611,7 +616,7 @@ new_library_button_clicked_cb (GtkWidget  *widget,
 
 	dialog = gth_request_dialog_new (GTK_WINDOW (data->dialog),
 					 GTK_DIALOG_MODAL,
-					 _("New library"),
+					 _("New Library"),
 					 _("Enter the library name:"),
 					 _GTK_LABEL_CANCEL,
 					 _("C_reate"));
@@ -629,9 +634,9 @@ update_sensitivity_cb (gpointer user_data)
 {
 	DialogData *data = user_data;
 
-	if (data->update_selectection_event != 0) {
-		g_source_remove (data->update_selectection_event);
-		data->update_selectection_event = 0;
+	if (data->update_selection_event != 0) {
+		g_source_remove (data->update_selection_event);
+		data->update_selection_event = 0;
 	}
 
 	update_sensitivity (data);
@@ -644,9 +649,17 @@ static void
 file_selection_changed_cb (GthFileSelection *self,
 			   DialogData       *data)
 {
-	if (data->update_selectection_event != 0)
-		g_source_remove (data->update_selectection_event);
-	data->update_selectection_event = g_timeout_add (UPDATE_SELECTION_DELAY, update_sensitivity_cb, data);
+	if (data->update_selection_event != 0)
+		g_source_remove (data->update_selection_event);
+	data->update_selection_event = g_timeout_add (UPDATE_SELECTION_DELAY, update_sensitivity_cb, data);
+}
+
+
+static void
+keep_open_button_toggled_cb (GtkToggleButton *button,
+			     DialogData      *data)
+{
+	update_sensitivity (data);
 }
 
 
@@ -654,8 +667,9 @@ void
 dlg_add_to_catalog (GthBrowser *browser)
 {
 	DialogData       *data;
-	GFile            *base;
 	GtkTreeSelection *selection;
+	char             *last_catalog;
+	GtkWidget        *sep;
 
 	if (gth_browser_get_dialog (browser, ADD_TO_CATALOG_DIALOG_NAME)) {
 		gtk_window_present (GTK_WINDOW (gth_browser_get_dialog (browser, ADD_TO_CATALOG_DIALOG_NAME)));
@@ -665,7 +679,43 @@ dlg_add_to_catalog (GthBrowser *browser)
 	data = g_new0 (DialogData, 1);
 	data->browser = browser;
 	data->builder = _gtk_builder_new_from_file ("add-to-catalog.ui", "catalogs");
-	data->dialog = _gtk_builder_get_widget (data->builder, "add_to_catalog_dialog");
+	data->settings = g_settings_new (GTHUMB_CATALOGS_SCHEMA);
+
+	data->dialog = g_object_new (GTK_TYPE_DIALOG,
+				     "title", _("Add to Catalog"),
+				     "transient-for", GTK_WINDOW (browser),
+				     "modal", FALSE,
+				     "use-header-bar", _gtk_settings_get_dialogs_use_header (),
+				     NULL);
+	gtk_container_set_border_width (GTK_CONTAINER (data->dialog), 5);
+
+	data->info = gth_file_selection_info_new ();
+	gtk_widget_show (data->info);
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (data->dialog))),
+			    data->info,
+			    FALSE,
+			    FALSE,
+			    0);
+
+	sep = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+	gtk_widget_show (sep);
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (data->dialog))),
+			    sep,
+			    FALSE,
+			    FALSE,
+			    5);
+
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (data->dialog))),
+			    GET_WIDGET ("dialog_content"),
+			    FALSE,
+			    FALSE,
+			    0);
+
+	gtk_dialog_add_buttons (GTK_DIALOG (data->dialog),
+				_GTK_LABEL_CLOSE, GTK_RESPONSE_CANCEL,
+				_("_Add"), GTK_RESPONSE_OK,
+				NULL);
+	_gtk_dialog_add_class_to_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK, GTK_STYLE_CLASS_SUGGESTED_ACTION);
 
 	gth_browser_set_dialog (browser, ADD_TO_CATALOG_DIALOG_NAME, data->dialog);
 
@@ -674,38 +724,34 @@ dlg_add_to_catalog (GthBrowser *browser)
 	data->add_data->parent_window = data->add_data->dialog = data->dialog;
 	add_data_ref (data->add_data);
 
-	base = g_file_new_for_uri ("catalog:///");
-	data->source_tree = gth_source_tree_new (base);
-	g_object_unref (base);
-
+	last_catalog = g_settings_get_string (data->settings, PREF_CATALOGS_LAST_CATALOG);
+	data->source_tree = gth_vfs_tree_new ("catalog:///", last_catalog);
 	gtk_widget_show (data->source_tree);
 	gtk_container_add (GTK_CONTAINER (GET_WIDGET ("catalog_list_scrolled_window")), data->source_tree);
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("catalogs_label")), data->source_tree);
 	update_sensitivity (data);
 
+	g_free (last_catalog);
+
 	/* Set the signals handlers. */
 
 	g_signal_connect (G_OBJECT (data->dialog),
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
-	g_signal_connect_swapped (G_OBJECT (GET_WIDGET ("cancel_button")),
+	g_signal_connect_swapped (gtk_dialog_get_widget_for_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_CANCEL),
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
 				  G_OBJECT (data->dialog));
 	g_signal_connect (G_OBJECT (data->source_tree),
-			  "open",
-			  G_CALLBACK (source_tree_open_cb),
+			  "changed",
+			  G_CALLBACK (source_tree_changed_cb),
 			  data);
-	g_signal_connect (G_OBJECT (GET_WIDGET ("add_button")),
+	g_signal_connect (gtk_dialog_get_widget_for_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK),
 			  "clicked",
 			  G_CALLBACK (add_button_clicked_cb),
 			  data);
-	g_signal_connect (G_OBJECT (GET_WIDGET ("add_close_button")),
-			  "clicked",
-			  G_CALLBACK (add_close_button_clicked_cb),
-			  data);
 	g_signal_connect (G_OBJECT (GET_WIDGET ("new_catalog_button")),
 			  "clicked",
 			  G_CALLBACK (new_catalog_button_clicked_cb),
@@ -714,7 +760,10 @@ dlg_add_to_catalog (GthBrowser *browser)
 			  "clicked",
 			  G_CALLBACK (new_library_button_clicked_cb),
 			  data);
-
+	g_signal_connect (G_OBJECT (GET_WIDGET ("keep_open_checkbutton")),
+			  "toggled",
+			  G_CALLBACK (keep_open_button_toggled_cb),
+			  data);
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->source_tree));
 	g_signal_connect (selection,
 			  "changed",
