@@ -26,7 +26,7 @@
 #include "gth-edit-metadata-dialog.h"
 
 
-#define UPDATE_SELECTION_DELAY 300
+#define UPDATE_SELECTION_DELAY 50
 
 
 typedef struct {
@@ -137,6 +137,8 @@ saver_completed_cb (GthTask  *task,
 
 	if (data->close_dialog)
 		close_dialog (data);
+	else
+		gth_browser_show_next_image (data->browser, FALSE, FALSE);
 
 	dialog_data_unref (data);
 	_g_object_unref (task);
@@ -154,7 +156,7 @@ edit_metadata_dialog__response_cb (GtkDialog *dialog,
 	GList      *scan;
 	GthTask    *task;
 
-	if ((response != GTK_RESPONSE_OK) && (response != GTK_RESPONSE_APPLY)) {
+	if (response != GTK_RESPONSE_OK) {
 		cancel_file_list_loading (data);
 		close_dialog (data);
 		return;
@@ -163,7 +165,7 @@ edit_metadata_dialog__response_cb (GtkDialog *dialog,
 	if (data->file_list == NULL)
 		return;
 
-	data->close_dialog = (response == GTK_RESPONSE_OK);
+	data->close_dialog = ! gth_edit_metadata_dialog_get_keep_open (GTH_EDIT_METADATA_DIALOG (data->dialog));
 
 	/* get the parents list */
 
@@ -271,6 +273,8 @@ update_file_list (gpointer user_data)
 	file_data_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (data->browser)), items);
 	loader_data->files = gth_file_data_list_to_file_list (file_data_list);
 
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK, loader_data->files != NULL);
+
 	data->loader = gth_load_file_data_task_new (loader_data->files, "*");
 	g_signal_connect (data->loader,
 			  "completed",
@@ -310,10 +314,20 @@ dlg_edit_metadata (GthBrowser *browser,
 	data = g_new0 (DialogData, 1);
 	data->ref = 1;
 	data->browser = browser;
-	data->dialog = g_object_new (dialog_type, 0);
+	data->dialog = g_object_new (dialog_type,
+				     "transient-for", GTK_WINDOW (browser),
+				     "modal", FALSE,
+				     "use-header-bar", _gtk_settings_get_dialogs_use_header (),
+				     NULL);
 	data->dialog_name = g_strdup (dialog_name);
 	data->never_shown = TRUE;
 
+	gtk_dialog_add_buttons (GTK_DIALOG (data->dialog),
+				_GTK_LABEL_CLOSE, GTK_RESPONSE_CANCEL,
+				_GTK_LABEL_SAVE, GTK_RESPONSE_OK,
+				NULL);
+
+	_gtk_dialog_add_class_to_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK, GTK_STYLE_CLASS_SUGGESTED_ACTION);
 	gth_browser_set_dialog (browser, data->dialog_name, data->dialog);
 
 	g_signal_connect (G_OBJECT (data->dialog),
