@@ -24,6 +24,7 @@
 #include <gtk/gtk.h>
 #include <gthumb.h>
 #include "gth-search-editor.h"
+#include "gth-search-source-selector.h"
 
 
 #define GET_WIDGET(name) _gtk_builder_get_widget (self->priv->builder, (name))
@@ -31,7 +32,6 @@
 
 struct _GthSearchEditorPrivate {
 	GtkBuilder *builder;
-	GtkWidget  *location_chooser;
 	GtkWidget  *match_type_combobox;
 };
 
@@ -69,7 +69,6 @@ gth_search_editor_init (GthSearchEditor *dialog)
 {
 	dialog->priv = gth_search_editor_get_instance_private (dialog);
 	dialog->priv->builder = NULL;
-	dialog->priv->location_chooser = NULL;
 	dialog->priv->match_type_combobox = NULL;
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (dialog), GTK_ORIENTATION_VERTICAL);
 }
@@ -78,15 +77,26 @@ gth_search_editor_init (GthSearchEditor *dialog)
 static void
 update_sensitivity (GthSearchEditor *self)
 {
-	GList *test_selectors;
+	GList *selectors;
 	int    more_selectors;
 	GList *scan;
 
-	test_selectors = gtk_container_get_children (GTK_CONTAINER (GET_WIDGET ("tests_box")));
-	more_selectors = (test_selectors != NULL) && (test_selectors->next != NULL);
-	for (scan = test_selectors; scan; scan = scan->next)
+	/* sources */
+
+	selectors = gtk_container_get_children (GTK_CONTAINER (GET_WIDGET ("sources_box")));
+	more_selectors = (selectors != NULL) && (selectors->next != NULL);
+	for (scan = selectors; scan; scan = scan->next)
+		gth_search_source_selector_can_remove (GTH_SEARCH_SOURCE_SELECTOR (scan->data), more_selectors);
+	g_list_free (selectors);
+
+	/* tests */
+
+	selectors = gtk_container_get_children (GTK_CONTAINER (GET_WIDGET ("tests_box")));
+	more_selectors = (selectors != NULL) && (selectors->next != NULL);
+	for (scan = selectors; scan; scan = scan->next)
 		gth_test_selector_can_remove (GTH_TEST_SELECTOR (scan->data), more_selectors);
-	g_list_free (test_selectors);
+	g_list_free (selectors);
+
 }
 
 
@@ -102,13 +112,6 @@ gth_search_editor_construct (GthSearchEditor *self,
     	gtk_container_set_border_width (GTK_CONTAINER (content), 0);
   	gtk_box_pack_start (GTK_BOX (self), content, TRUE, TRUE, 0);
 
-	self->priv->location_chooser = g_object_new (GTH_TYPE_LOCATION_CHOOSER,
-						     "show-entry-points", TRUE,
-						     "relief", GTK_RELIEF_NORMAL,
-						     NULL);
-	gtk_widget_show (self->priv->location_chooser);
-  	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("location_box")), self->priv->location_chooser, TRUE, TRUE, 0);
-
 	self->priv->match_type_combobox = gtk_combo_box_text_new ();
   	_gtk_combo_box_append_texts (GTK_COMBO_BOX_TEXT (self->priv->match_type_combobox),
   				     _("all the following rules"),
@@ -121,7 +124,6 @@ gth_search_editor_construct (GthSearchEditor *self,
 
 	gtk_label_set_use_underline (GTK_LABEL (GET_WIDGET ("match_label")), TRUE);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("match_label")), self->priv->match_type_combobox);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("start_at_label")), self->priv->location_chooser);
 
   	gth_search_editor_set_search (self, search);
 }
@@ -194,17 +196,71 @@ _gth_search_editor_add_test (GthSearchEditor *self,
 }
 
 
+static GtkWidget *
+_gth_search_editor_add_source (GthSearchEditor *self,
+			       int              pos);
+
+
 static void
-_gth_search_editor_set_new_search (GthSearchEditor *self)
+test_selector_add_source_cb (GthTestSelector *selector,
+			     GthSearchEditor *self)
 {
-	GFile *home_location;
+	int pos;
 
-	home_location = g_file_new_for_uri (get_home_uri ());
-	gth_location_chooser_set_current (GTH_LOCATION_CHOOSER (self->priv->location_chooser), home_location);
-	g_object_unref (home_location);
+	pos = _gtk_container_get_pos (GTK_CONTAINER (GET_WIDGET ("sources_box")), (GtkWidget*) selector);
+	_gth_search_editor_add_source (self, pos == -1 ? -1 : pos + 1);
+	update_sensitivity (self);
+}
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("include_subfolders_checkbutton")), TRUE);
-	_gtk_container_remove_children (GTK_CONTAINER (GET_WIDGET ("tests_box")), NULL, NULL);
+
+static void
+test_selector_remove_source_cb (GthTestSelector *selector,
+				GthSearchEditor *self)
+{
+	gtk_container_remove (GTK_CONTAINER (GET_WIDGET ("sources_box")), (GtkWidget*) selector);
+	update_sensitivity (self);
+}
+
+
+static GtkWidget *
+_gth_search_editor_add_source (GthSearchEditor *self,
+			       int              pos)
+{
+	GthSearchSource *source;
+	GtkWindow       *window;
+	GtkWidget       *source_selector;
+
+	source = NULL;
+	window = _gtk_widget_get_toplevel_if_window (GTK_WIDGET (self));
+	if (window != NULL)
+		window = gtk_window_get_transient_for (window);
+	if ((window != NULL) && GTH_IS_BROWSER (window)) {
+		source = gth_search_source_new ();
+		gth_search_source_set_folder (source, gth_browser_get_location (GTH_BROWSER (window)));
+		gth_search_source_set_recursive (source, TRUE);
+	}
+	source_selector = gth_search_source_selector_new_with_source (source);
+	gtk_widget_show (source_selector);
+
+	g_signal_connect (G_OBJECT (source_selector),
+			  "add_source",
+			  G_CALLBACK (test_selector_add_source_cb),
+			  self);
+	g_signal_connect (G_OBJECT (source_selector),
+			  "remove_source",
+			  G_CALLBACK (test_selector_remove_source_cb),
+			  self);
+
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("sources_box")), source_selector, FALSE, FALSE, 0);
+
+	if (pos >= 0)
+		gtk_box_reorder_child (GTK_BOX (GET_WIDGET ("sources_box")),
+				       source_selector,
+				       pos);
+
+	_g_object_unref (source);
+
+	return source_selector;
 }
 
 
@@ -212,40 +268,60 @@ void
 gth_search_editor_set_search (GthSearchEditor *self,
 			      GthSearch       *search)
 {
-	GthTestChain *test;
-	GthMatchType  match_type;
+	int           n_sources;
+	int           n_tests;
+	GthMatchType  match_type = GTH_MATCH_TYPE_NONE;
+	GList        *scan;
 
-	_gth_search_editor_set_new_search (self);
+	/* sources */
 
-	if (search == NULL) {
-		_gth_search_editor_add_test (self, -1);
-		update_sensitivity (self);
-		return;
-	}
+	_gtk_container_remove_children (GTK_CONTAINER (GET_WIDGET ("sources_box")), NULL, NULL);
+	n_sources = 0;
 
-	gth_location_chooser_set_current (GTH_LOCATION_CHOOSER (self->priv->location_chooser), gth_search_get_folder (search));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("include_subfolders_checkbutton")), gth_search_is_recursive (search));
+	if (search != NULL) {
+		for (scan = gth_search_get_sources (search); scan; scan = scan->next) {
+			GthSearchSource *source = scan->data;
+			GtkWidget       *source_selector;
 
-	test = gth_search_get_test (search);
-	match_type = (test != NULL) ? gth_test_chain_get_match_type (test) : GTH_MATCH_TYPE_NONE;
-	_gtk_container_remove_children (GTK_CONTAINER (GET_WIDGET ("tests_box")), NULL, NULL);
-	if (match_type != GTH_MATCH_TYPE_NONE) {
-		GList *tests;
-		GList *scan;
-
-		tests = gth_test_chain_get_tests (test);
-		for (scan = tests; scan; scan = scan->next) {
-			GthTest   *test = scan->data;
-			GtkWidget *test_selector;
-
-			test_selector = _gth_search_editor_add_test (self, -1);
-			gth_test_selector_set_test (GTH_TEST_SELECTOR (test_selector), test);
+			source_selector = _gth_search_editor_add_source (self, -1);
+			gth_search_source_selector_set_source (GTH_SEARCH_SOURCE_SELECTOR (source_selector), source);
+			n_sources += 1;
 		}
-		_g_object_list_unref (tests);
 	}
-	else
-		_gth_search_editor_add_test (self, -1);
 
+	/* tests */
+
+	_gtk_container_remove_children (GTK_CONTAINER (GET_WIDGET ("tests_box")), NULL, NULL);
+	n_tests = 0;
+
+	if (search != NULL) {
+		GthTestChain *test;
+
+		test = gth_search_get_test (search);
+		if (test != NULL)
+			match_type = gth_test_chain_get_match_type (test);
+
+		if (match_type != GTH_MATCH_TYPE_NONE) {
+			GList *tests;
+			GList *scan;
+
+			tests = gth_test_chain_get_tests (test);
+			for (scan = tests; scan; scan = scan->next) {
+				GthTest   *test = scan->data;
+				GtkWidget *test_selector;
+
+				test_selector = _gth_search_editor_add_test (self, -1);
+				gth_test_selector_set_test (GTH_TEST_SELECTOR (test_selector), test);
+				n_tests += 1;
+			}
+			_g_object_list_unref (tests);
+		}
+	}
+
+	if (n_sources == 0)
+		_gth_search_editor_add_source (self, -1);
+	if (n_tests == 0)
+		_gth_search_editor_add_test (self, -1);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (self->priv->match_type_combobox), (match_type == GTH_MATCH_TYPE_ANY) ? 1 : 0);
 
 	update_sensitivity (self);
@@ -257,22 +333,31 @@ gth_search_editor_get_search (GthSearchEditor  *self,
 			      GError          **error)
 {
 	GthSearch *search;
-	GFile     *folder;
+	GList     *sources;
 	GthTest   *test;
-	GList     *test_selectors;
+	GList     *selectors;
 	GList     *scan;
 
 	search = gth_search_new ();
 
-	folder = gth_location_chooser_get_current (GTH_LOCATION_CHOOSER (self->priv->location_chooser));
-	if (folder != NULL)
-		gth_search_set_folder (search, folder);
+	/* sources */
 
-	gth_search_set_recursive (search, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("include_subfolders_checkbutton"))));
+	sources = NULL;
+	selectors = gtk_container_get_children (GTK_CONTAINER (GET_WIDGET ("sources_box")));
+	for (scan = selectors; scan; scan = scan->next) {
+		GthSearchSourceSelector *selector = GTH_SEARCH_SOURCE_SELECTOR (scan->data);
+		sources = g_list_prepend (sources, gth_search_source_selector_get_source (selector));
+	}
+	g_list_free (selectors);
+	sources = g_list_reverse (sources);
+	gth_search_set_sources (search, sources);
+	_g_object_list_unref (sources);
+
+	/* tests */
 
 	test = gth_test_chain_new (gtk_combo_box_get_active (GTK_COMBO_BOX (self->priv->match_type_combobox)) + 1, NULL);
-	test_selectors = gtk_container_get_children (GTK_CONTAINER (GET_WIDGET ("tests_box")));
-	for (scan = test_selectors; scan; scan = scan->next) {
+	selectors = gtk_container_get_children (GTK_CONTAINER (GET_WIDGET ("tests_box")));
+	for (scan = selectors; scan; scan = scan->next) {
 		GthTestSelector *test_selector = GTH_TEST_SELECTOR (scan->data);
 		GthTest         *sub_test;
 
@@ -285,8 +370,9 @@ gth_search_editor_get_search (GthSearchEditor  *self,
 		gth_test_chain_add_test (GTH_TEST_CHAIN (test), sub_test);
 		g_object_unref (sub_test);
 	}
-	g_list_free (test_selectors);
+	g_list_free (selectors);
 	gth_search_set_test (search, GTH_TEST_CHAIN (test));
+	g_object_unref (test);
 
 	return search;
 }
