@@ -33,86 +33,59 @@
 #include "gth-folder-tree.h"
 #include "gth-main.h"
 #include "gth-preferences.h"
+#include "gth-shortcuts-window.h"
 #include "gth-sidebar.h"
 #include "gtk-utils.h"
 #include "gth-viewer-page.h"
 #include "main.h"
 
 
-static GtkWidget *
-_gth_application_get_current_window (GApplication *application)
+void
+gth_browser_activate_new_window (GSimpleAction *action,
+				 GVariant      *parameter,
+				 gpointer       user_data)
 {
-        GList *windows;
+	GtkWidget *browser = user_data;
+	GtkWidget *window;
 
-        windows = gtk_application_get_windows (GTK_APPLICATION (application));
-        if (windows == NULL)
-        	return NULL;
-
-        return GTK_WIDGET (windows->data);
+	window = gth_browser_new (gth_browser_get_location (GTH_BROWSER (browser)), NULL);
+	gtk_window_present (GTK_WINDOW (window));
 }
 
 
 void
-gth_application_activate_new_window (GSimpleAction *action,
-				     GVariant      *parameter,
-				     gpointer       user_data)
+gth_browser_activate_preferences (GSimpleAction *action,
+				  GVariant      *parameter,
+				  gpointer       user_data)
 {
-        GApplication *application = user_data;
-        GtkWidget    *browser;
-        GtkWidget    *window;
-
-        browser = _gth_application_get_current_window (application);
-        window = gth_browser_new (gth_browser_get_location (GTH_BROWSER (browser)), NULL);
-        gtk_window_present (GTK_WINDOW (window));
+	dlg_preferences (GTH_BROWSER (user_data));
 }
 
 
 void
-gth_application_activate_preferences (GSimpleAction *action,
-				      GVariant      *parameter,
-				      gpointer       user_data)
-{
-        GApplication *application = user_data;
-        GtkWidget    *browser;
-
-        browser = _gth_application_get_current_window (application);
-        dlg_preferences (GTH_BROWSER (browser));
-}
-
-
-void
-gth_application_activate_show_help (GSimpleAction *action,
-				    GVariant      *parameter,
-				    gpointer       user_data)
-{
-        GApplication *application = user_data;
-        GtkWidget    *browser;
-
-        browser = _gth_application_get_current_window (application);
-        show_help_dialog (GTK_WINDOW (browser), NULL);
-}
-
-
-void
-gth_application_activate_show_shortcuts (GSimpleAction *action,
-					 GVariant      *parameter,
-					 gpointer       user_data)
-{
-        GApplication *application = user_data;
-        GtkWidget    *browser;
-
-        browser = _gth_application_get_current_window (application);
-        show_help_dialog (GTK_WINDOW (browser), "gthumb-shortcuts");
-}
-
-
-void
-gth_application_activate_about (GSimpleAction *action,
+gth_browser_activate_show_help (GSimpleAction *action,
 				GVariant      *parameter,
 				gpointer       user_data)
 {
-        GApplication *application = user_data;
-	GthWindow    *window;
+	show_help_dialog (GTK_WINDOW (user_data), NULL);
+}
+
+
+void
+gth_browser_activate_show_shortcuts (GSimpleAction *action,
+				     GVariant      *parameter,
+				     gpointer       user_data)
+{
+	gth_shortcuts_window_new (GTH_WINDOW (user_data));
+}
+
+
+void
+gth_browser_activate_about (GSimpleAction *action,
+			    GVariant      *parameter,
+			    gpointer       user_data)
+{
+	GthBrowser   *browser = user_data;
 	const char   *authors[] = {
 #include "AUTHORS.tab"
 		NULL
@@ -137,21 +110,20 @@ gth_application_activate_about (GSimpleAction *action,
 	};
 	GdkPixbuf *logo;
 
-	window = (GthWindow *) _gth_application_get_current_window (application);
 	license_text = g_strconcat (_(license[0]), "\n\n",
 				    _(license[1]), "\n\n",
 				    _(license[2]),
 				    NULL);
 
-	logo = gtk_icon_theme_load_icon (gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (window))),
+	logo = gtk_icon_theme_load_icon (gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (browser))),
 					 "org.gnome.gThumb",
 					 128,
 					 0,
 					 NULL);
 
-	gtk_show_about_dialog (GTK_WINDOW (window),
+	gtk_show_about_dialog (GTK_WINDOW (browser),
 			       "version", PACKAGE_VERSION,
-			       "copyright", "Copyright \xc2\xa9 2001-2013 Free Software Foundation, Inc.",
+			       "copyright", "Copyright \xc2\xa9 2001-2020 Free Software Foundation, Inc.",
 			       "comments", _("An image viewer and browser for GNOME."),
 			       "authors", authors,
 			       "documenters", documenters,
@@ -168,16 +140,11 @@ gth_application_activate_about (GSimpleAction *action,
 
 
 void
-gth_application_activate_quit (GSimpleAction *action,
-			       GVariant      *parameter,
-			       gpointer       user_data)
+gth_browser_activate_quit (GSimpleAction *action,
+			   GVariant      *parameter,
+			   gpointer       user_data)
 {
-        GApplication *application = user_data;
-        GList        *windows;
-
-        windows = gtk_application_get_windows (GTK_APPLICATION (application));
-        if (windows != NULL)
-        	gth_quit (FALSE);
+	gth_quit (FALSE);
 }
 
 
@@ -377,6 +344,26 @@ gth_browser_activate_save_as (GSimpleAction *action,
 }
 
 
+void
+gth_browser_activate_toggle_edit_file (GSimpleAction *action,
+					GVariant      *state,
+					gpointer       user_data)
+{
+	GthBrowser *browser = user_data;
+	gth_browser_toggle_viewer_tools (browser);
+}
+
+
+void
+gth_browser_activate_toggle_file_properties (GSimpleAction *action,
+					     GVariant      *state,
+					     gpointer       user_data)
+{
+	GthBrowser *browser = user_data;
+	gth_browser_toggle_file_properties (browser);
+}
+
+
 void
 gth_browser_activate_viewer_edit_file (GSimpleAction *action,
 				       GVariant      *state,
@@ -475,6 +462,19 @@ gth_browser_activate_show_statusbar (GSimpleAction *action,
 }
 
 
+void
+gth_browser_activate_toggle_statusbar (GSimpleAction *action,
+				       GVariant      *state,
+				       gpointer       user_data)
+{
+	GSettings *settings;
+
+	settings = g_settings_new (GTHUMB_BROWSER_SCHEMA);
+	g_settings_set_boolean (settings, PREF_BROWSER_STATUSBAR_VISIBLE, ! g_settings_get_boolean (settings, PREF_BROWSER_STATUSBAR_VISIBLE));
+	g_object_unref (settings);
+}
+
+
 void
 gth_browser_activate_show_sidebar (GSimpleAction *action,
 				   GVariant      *state,
@@ -490,6 +490,19 @@ gth_browser_activate_show_sidebar (GSimpleAction *action,
 }
 
 
+void
+gth_browser_activate_toggle_sidebar (GSimpleAction *action,
+				     GVariant      *state,
+				     gpointer       user_data)
+{
+	GSettings *settings;
+
+	settings = g_settings_new (GTHUMB_BROWSER_SCHEMA);
+	g_settings_set_boolean (settings, PREF_BROWSER_SIDEBAR_VISIBLE, ! g_settings_get_boolean (settings, PREF_BROWSER_SIDEBAR_VISIBLE));
+	g_object_unref (settings);
+}
+
+
 void
 gth_browser_activate_show_thumbnail_list (GSimpleAction *action,
 					  GVariant      *state,
@@ -505,6 +518,39 @@ gth_browser_activate_show_thumbnail_list (GSimpleAction *action,
 }
 
 
+void
+gth_browser_activate_toggle_thumbnail_list (GSimpleAction *action,
+					    GVariant      *state,
+					    gpointer       user_data)
+{
+	GSettings *settings;
+
+	settings = g_settings_new (GTHUMB_BROWSER_SCHEMA);
+	g_settings_set_boolean (settings, PREF_BROWSER_THUMBNAIL_LIST_VISIBLE, ! g_settings_get_boolean (settings, PREF_BROWSER_THUMBNAIL_LIST_VISIBLE));
+	g_object_unref (settings);
+}
+
+
+void
+gth_browser_activate_show_first_image (GSimpleAction *action,
+				       GVariant      *state,
+				       gpointer       user_data)
+{
+	GthBrowser *browser = GTH_BROWSER (user_data);
+	gth_browser_show_first_image (browser, FALSE, FALSE);
+}
+
+
+void
+gth_browser_activate_show_last_image (GSimpleAction *action,
+				      GVariant      *state,
+				      gpointer       user_data)
+{
+	GthBrowser *browser = GTH_BROWSER (user_data);
+	gth_browser_show_last_image (browser, FALSE, FALSE);
+}
+
+
 void
 gth_browser_activate_show_previous_image (GSimpleAction *action,
 					  GVariant      *state,
@@ -533,3 +579,38 @@ gth_browser_activate_apply_editor_changes (GSimpleAction *action,
 	GthBrowser *browser = GTH_BROWSER (user_data);
 	gth_browser_apply_editor_changes (browser);
 }
+
+
+void
+gth_browser_activate_select_all (GSimpleAction *action,
+				 GVariant      *state,
+				 gpointer       user_data)
+{
+	GthBrowser *browser = GTH_BROWSER (user_data);
+	GtkWidget  *file_view;
+
+	file_view = gth_browser_get_file_list_view (browser);
+	gth_file_selection_select_all (GTH_FILE_SELECTION (file_view));
+}
+
+
+void
+gth_browser_activate_unselect_all (GSimpleAction *action,
+				   GVariant      *state,
+				   gpointer       user_data)
+{
+	GthBrowser *browser = GTH_BROWSER (user_data);
+	GtkWidget  *file_view;
+
+	file_view = gth_browser_get_file_list_view (browser);
+	gth_file_selection_unselect_all (GTH_FILE_SELECTION (file_view));
+}
+
+
+void
+gth_browser_activate_show_menu (GSimpleAction *action,
+				GVariant      *state,
+				gpointer       user_data)
+{
+	gth_browser_show_menu (GTH_BROWSER (user_data));
+}
