@@ -30,20 +30,6 @@
 #include "gth-main.h"
 
 
-SizeValue
-ImageSizeValues[IMAGE_SIZE_N] = {
-		{ 320, 200 },
-		{ 320, 320 },
-		{ 640, 480 },
-		{ 640, 640 },
-		{ 800, 600 },
-		{ 800, 800 },
-		{ 1024, 768 },
-		{ 1024, 1024 },
-		{ 1280, 960 },
-		{ 1280, 1280 } };
-
-
 gboolean
 scale_keeping_ratio_min (int      *width,
 			 int      *height,
@@ -162,6 +148,13 @@ _g_buffer_resize_image (void          *buffer,
 
 	surface = gth_image_get_cairo_surface (image);
 	scaled = _cairo_image_surface_scale (surface, width, height, SCALE_FILTER_BEST, NULL);
+	if (scaled == NULL) {
+		cairo_surface_destroy (surface);
+		g_object_unref (image);
+		g_object_unref (istream);
+		return FALSE;
+	}
+
 	gth_image_set_cairo_surface (image, scaled);
 	result = gth_image_save_to_buffer (image,
 					   mime_type,
