@@ -21,6 +21,8 @@
 
 #include <config.h>
 #include <gtk/gtk.h>
+#include "glib-utils.h"
+#include "gth-main.h"
 #include "gth-window.h"
 #include "gth-window-title.h"
 #include "gtk-utils.h"
@@ -54,6 +56,9 @@ struct _GthWindowPrivate {
 	GthWindowSize   *window_size;
 	GtkWindowGroup  *window_group;
 	GtkAccelGroup   *accel_group;
+	GHashTable      *shortcuts;
+	GPtrArray       *shortcuts_v;
+	GHashTable      *shortcut_groups;
 };
 
 
@@ -249,6 +254,9 @@ gth_window_finalize (GObject *object)
 	g_free (window->priv->window_size);
 	g_object_unref (window->priv->window_group);
 	g_object_unref (window->priv->accel_group);
+	g_hash_table_unref (window->priv->shortcuts);
+	g_ptr_array_unref (window->priv->shortcuts_v);
+	g_hash_table_unref (window->priv->shortcut_groups);
 
 	G_OBJECT_CLASS (gth_window_parent_class)->finalize (object);
 }
@@ -399,6 +407,11 @@ gth_window_init (GthWindow *window)
 	window->priv->accel_group = gtk_accel_group_new ();
 	gtk_window_add_accel_group (GTK_WINDOW (window), window->priv->accel_group);
 
+	window->priv->shortcuts = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	window->priv->shortcuts_v = g_ptr_array_new_with_free_func ((GDestroyNotify) gth_shortcut_free);
+
+	window->priv->shortcut_groups = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_ptr_array_unref);
+
 	gtk_window_set_application (GTK_WINDOW (window), Main_Application);
 }
 
@@ -692,6 +705,26 @@ gth_window_get_accel_group (GthWindow *window)
 }
 
 
+static void
+_gth_window_add_shortcut (GthWindow   *window,
+			  GthShortcut *shortcut)
+{
+	g_ptr_array_add (window->priv->shortcuts_v, shortcut);
+	g_hash_table_insert (window->priv->shortcuts,
+			     g_strdup (shortcut->detailed_action),
+			     shortcut);
+}
+
+
+static void
+_gth_window_remove_shortcut (GthWindow   *window,
+			     GthShortcut *shortcut)
+{
+	g_hash_table_remove (window->priv->shortcuts, shortcut->detailed_action);
+	g_ptr_array_remove (window->priv->shortcuts_v, shortcut);
+}
+
+
 void
 gth_window_add_accelerators (GthWindow			*window,
 			     const GthAccelerator	*accelerators,
@@ -703,12 +736,19 @@ gth_window_add_accelerators (GthWindow			*window,
 	accel_group = gth_window_get_accel_group (window);
 	for (i = 0; i < n_accelerators; i++) {
 		const GthAccelerator *acc = accelerators + i;
+		GthShortcut          *shortcut;
 
 		_gtk_window_add_accelerator_for_action (GTK_WINDOW (window),
 							accel_group,
 							acc->action_name,
 							acc->accelerator,
 							NULL);
+
+		shortcut = gth_shortcut_new (acc->action_name, NULL);
+		shortcut->context = GTH_SHORTCUT_CONTEXT_INTERNAL | GTH_SHORTCUT_CONTEXT_BROWSER_VIEWER;
+		shortcut->category = GTH_SHORTCUT_CATEGORY_HIDDEN;
+		gth_shortcut_set_accelerator (shortcut, acc->accelerator);
+		_gth_window_add_shortcut (window, shortcut);
 	}
 }
 
@@ -765,3 +805,286 @@ gth_window_change_action_state (GthWindow  *window,
 		g_variant_unref (old_state);
 	g_variant_unref (new_state);
 }
+
+
+void
+gth_window_add_shortcuts (GthWindow         *window,
+			  const GthShortcut *shortcuts,
+			  int                n_shortcuts)
+{
+	int i;
+
+	for (i = 0; i < n_shortcuts; i++) {
+		const GthShortcut *shortcut = shortcuts + i;
+		GthShortcut       *new_shortcut;
+
+		new_shortcut = gth_shortcut_dup (shortcut);
+		gth_shortcut_set_accelerator (new_shortcut, shortcut->default_accelerator);
+
+		_gth_window_add_shortcut (window, new_shortcut);
+	}
+}
+
+
+GPtrArray *
+gth_window_get_shortcuts (GthWindow *window)
+{
+	g_return_val_if_fail (GTH_IS_WINDOW (window), NULL);
+
+	return window->priv->shortcuts_v;
+}
+
+
+GthShortcut *
+gth_window_get_shortcut (GthWindow  *window,
+			 const char *detailed_action)
+{
+	g_return_val_if_fail (GTH_IS_WINDOW (window), NULL);
+
+	return g_hash_table_lookup (window->priv->shortcuts, detailed_action);
+}
+
+
+static int
+sort_shortcuts_by_category (gconstpointer a,
+			    gconstpointer b)
+{
+	const GthShortcut   *sa = * (GthShortcut **) a;
+	const GthShortcut   *sb = * (GthShortcut **) b;
+	int                  result;
+	GthShortcutCategory *cat_a;
+	GthShortcutCategory *cat_b;
+
+	result = 0;
+	cat_a = gth_main_get_shortcut_category (sa->category);
+	cat_b = gth_main_get_shortcut_category (sb->category);
+	if ((cat_a != NULL) && (cat_b != NULL)) {
+		if (cat_a->sort_order < cat_b->sort_order)
+			result = -1;
+		else if (cat_a->sort_order > cat_b->sort_order)
+			result = 1;
+	}
+	if (result == 0)
+		result = g_strcmp0 (sa->description, sb->description);
+
+	return result;
+}
+
+
+GPtrArray *
+gth_window_get_shortcuts_by_category (GthWindow *window)
+{
+	g_return_val_if_fail (GTH_IS_WINDOW (window), NULL);
+
+	g_ptr_array_sort (window->priv->shortcuts_v, sort_shortcuts_by_category);
+	return window->priv->shortcuts_v;
+}
+
+
+gboolean
+gth_window_activate_shortcut (GthWindow       *window,
+			      int              context,
+			      guint            keycode,
+			      GdkModifierType  modifiers)
+{
+	gboolean     activated = FALSE;
+	GthShortcut *shortcut;
+
+	modifiers = modifiers & gtk_accelerator_get_default_mod_mask ();
+	shortcut = gth_shortcut_array_find (window->priv->shortcuts_v, context, keycode, modifiers);
+	if (shortcut != NULL) {
+		GAction *action;
+
+		if ((shortcut->context & GTH_SHORTCUT_CONTEXT_INTERNAL) != 0)
+			return FALSE;
+
+		if ((shortcut->context & GTH_SHORTCUT_CONTEXT_DOC) != 0)
+			return FALSE;
+
+		action = g_action_map_lookup_action (G_ACTION_MAP (window), shortcut->action_name);
+		if (action != NULL) {
+			g_action_activate (action, shortcut->action_parameter);
+			activated = TRUE;
+		}
+	}
+
+	return activated;
+}
+
+
+void
+gth_window_load_shortcuts (GthWindow *window)
+{
+	g_return_if_fail (GTH_IS_WINDOW (window));
+
+	gth_shortcuts_load_from_file (window->priv->shortcuts_v,
+				      window->priv->shortcuts,
+				      NULL);
+}
+
+
+void
+gth_window_add_removable_shortcut (GthWindow   *window,
+				   const char  *group_name,
+				   GthShortcut *shortcut)
+{
+	GPtrArray   *shortcuts_v;
+	GthShortcut *old_shortcut;
+	GthShortcut *new_shortcut;
+
+	g_return_if_fail (GTH_IS_WINDOW (window));
+	g_return_if_fail (group_name != NULL);
+	g_return_if_fail (shortcut != NULL);
+	g_return_if_fail (shortcut->detailed_action != NULL);
+
+	/* create the group if it doesn't exist. */
+
+	shortcuts_v = g_hash_table_lookup (window->priv->shortcut_groups, group_name);
+	if (shortcuts_v == NULL) {
+		shortcuts_v = g_ptr_array_new ();
+		g_hash_table_insert (window->priv->shortcut_groups,
+				     g_strdup (group_name),
+				     shortcuts_v);
+	}
+
+	/* remove the old shortcut */
+
+	old_shortcut = g_hash_table_lookup (window->priv->shortcuts, shortcut->detailed_action);
+	if (old_shortcut != NULL) {
+		g_ptr_array_remove (shortcuts_v, old_shortcut);
+		_gth_window_remove_shortcut (window, old_shortcut);
+	}
+
+	/* add the new shortcut */
+
+	new_shortcut = gth_shortcut_dup (shortcut);
+	_gth_window_add_shortcut (window, new_shortcut);
+	g_ptr_array_add (shortcuts_v, new_shortcut);
+}
+
+
+void
+gth_window_remove_shortcuts (GthWindow  *window,
+			     const char *group_name)
+{
+	GPtrArray *shortcuts_v;
+	int        i;
+
+	g_return_if_fail (GTH_IS_WINDOW (window));
+	g_return_if_fail (group_name != NULL);
+
+	shortcuts_v = g_hash_table_lookup (window->priv->shortcut_groups, group_name);
+	if (shortcuts_v == NULL)
+		return;
+
+	for (i = 0; i < shortcuts_v->len; i++) {
+		GthShortcut *shortcut = g_ptr_array_index (shortcuts_v, i);
+		_gth_window_remove_shortcut (window, shortcut);
+	}
+
+	g_hash_table_remove (window->priv->shortcut_groups, group_name);
+}
+
+
+gboolean
+gth_window_can_change_shortcut (GthWindow         *window,
+				const char        *detailed_action,
+				int                context,
+				guint              keycode,
+				GdkModifierType    modifiers,
+				GtkWindow         *parent)
+{
+	GthShortcut *shortcut;
+
+	if (window == NULL)
+		return TRUE;
+
+	shortcut = gth_shortcut_array_find (gth_window_get_shortcuts (window ),
+					    context,
+					    keycode,
+					    modifiers);
+
+	if (shortcut == NULL)
+		return TRUE;
+
+	if (g_strcmp0 (shortcut->detailed_action, detailed_action) == 0)
+		return FALSE;
+
+	if (gth_shortcut_customizable (shortcut)) {
+		char      *label;
+		char      *msg;
+		GtkWidget *dialog;
+		gboolean   reassign;
+
+		label = gtk_accelerator_get_label (keycode, modifiers);
+		msg = g_strdup_printf (_("The key combination «%s» is already assigned to the action «%s».  Do you want to reassign it to this action instead?"),
+				       label,
+				       shortcut->description);
+
+		dialog = _gtk_yesno_dialog_new (parent,
+						GTK_DIALOG_MODAL,
+						msg,
+						_GTK_LABEL_CANCEL,
+						_("Reassign"));
+
+		reassign = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES;
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+
+		g_free (msg);
+		g_free (label);
+
+		if (! reassign)
+			return FALSE;
+	}
+	else {
+		char      *label;
+		char      *msg;
+		GtkWidget *dialog;
+
+		label = gtk_accelerator_get_label (keycode, modifiers);
+		if (shortcut->description != NULL)
+			msg = g_strdup_printf (_("The key combination «%s» is already assigned to the action «%s» and cannot be changed."),
+					       label,
+					       shortcut->description);
+		else
+			msg = g_strdup_printf (_("The key combination «%s» is already assigned and cannot be changed."),
+					       label);
+
+		dialog = _gtk_message_dialog_new (parent,
+						  GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+						  _GTK_ICON_NAME_DIALOG_ERROR,
+						  NULL,
+						  msg,
+						  _GTK_LABEL_OK, GTK_RESPONSE_OK,
+						  NULL);
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+
+		g_free (msg);
+		g_free (label);
+
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
+void
+gth_window_copy_shortcuts (GthWindow *to_window,
+			   GthWindow *from_window,
+			   int        context)
+{
+	int i;
+
+	for (i = 0; i < from_window->priv->shortcuts_v->len; i++) {
+		const GthShortcut *shortcut = g_ptr_array_index (from_window->priv->shortcuts_v, i);
+		GthShortcut       *new_shortcut;
+
+		if ((shortcut->context & context) == 0)
+			continue;
+
+		new_shortcut = gth_shortcut_dup (shortcut);
+		_gth_window_add_shortcut (to_window, new_shortcut);
+	}
+}
