@@ -51,6 +51,31 @@ typedef struct {
 	const char *accelerator;
 } GthAccelerator;
 
+
+typedef enum /*< skip >*/ {
+	IMAGE_SIZE_320x200,
+	IMAGE_SIZE_320x320,
+	IMAGE_SIZE_640x480,
+	IMAGE_SIZE_640x640,
+	IMAGE_SIZE_800x600,
+	IMAGE_SIZE_800x800,
+	IMAGE_SIZE_1024x768,
+	IMAGE_SIZE_1024x1024,
+	IMAGE_SIZE_1280x960,
+	IMAGE_SIZE_1280x1280,
+	IMAGE_SIZE_N
+} ImageSize;
+
+
+typedef struct {
+	int width;
+	int height;
+} SizeValue;
+
+
+extern SizeValue ImageSizeValues[IMAGE_SIZE_N];
+
+
 GtkWidget *     _gtk_message_dialog_new                    (GtkWindow        *parent,
 							    GtkDialogFlags    flags,
 							    const char       *icon_name,
@@ -121,10 +146,13 @@ void            _gtk_widget_set_margin                     (GtkWidget        *wi
 							    int               bottom,
 							    int               left);
 void            _gtk_tree_path_list_free                   (GList            *list);
+GtkTreePath *   _gtk_tree_path_get_previous_or_parent      (GtkTreePath      *path);
 int             _gtk_paned_get_position2                   (GtkPaned         *paned);
 void            _g_launch_command                          (GtkWidget        *parent,
 					    	    	    const char       *command,
 					    	    	    const char       *name,
+							    GAppInfoCreateFlags
+							                      flags,
 					    	    	    GList            *files);
 void            _gtk_window_resize_to_fit_screen_height    (GtkWidget        *window,
 					    	    	    int               default_width);
@@ -166,6 +194,7 @@ GdkCursor *     _gdk_cursor_new_for_widget                 (GtkWidget
 							    GdkCursorType        cursor_type);
 void		_gtk_widget_reparent			   (GtkWidget		*widget,
 							    GtkWidget		*new_parent);
+GtkWindow *     _gtk_widget_get_toplevel_if_window         (GtkWidget           *widget);
 
 G_END_DECLS
 
